#
# fishing.R, 16 May 21
#
# Data from:
# Composing effective software security assurance workflows.
# W. R. Nichols, J. D. McHale, D. Sweeney, W. Snavely, and A. Volkman.

library(colorspace)
library(plyr)

par(bty="l")
par(las=1)
par(pch="+")

pal_col=rainbow(2)

# Fit models to the inspection estimates
# Construction phases
c_phase=c("Ident", "Reqts", "HLD", "Design", "Code", "Test Devel", "Documentation")

tf=read.csv("CESAW_task_fact.csv.xz", as.is=TRUE)

tf$start_date=as.Date(tf$task_actual_start_date, format="%m/%d/%Y")
tf$end_date=as.Date(tf$task_actual_complete_date, format="%m/%d/%Y")

# Projects with the most rows
# p95=subset(tf, project_key == 95)   # 2nd
# p614=subset(tf, project_key == 614) # 4th
p615=subset(tf, project_key == 615) # 1st
# p617=subset(tf, project_key == 617) # 3rd

# Remove outliers
# p614=subset(p614, start_date > as.Date("2014-06-18"))
p615=subset(p615, start_date > as.Date("2010-01-01"))

proj_range=c(0, 1+length(unique(tf$project_key)))

# Fit models to the inspection estimates

inspect_est=function(ins_str, proj_df)
{
ins_df=subset(proj_df, phase_short_name == ins_str)
ins_df=subset(ins_df, (task_actual_time_minutes > 0) &
                        (task_plan_time_minutes >0))

if (nrow(ins_df) < 2)
   return(NULL)
# Require that a person make some minimum number of estimates
pk=count(ins_df$person_key)
many_pk=subset(pk, freq > 4)
ins_df=subset(ins_df, person_key %in% many_pk$x)

# print(c(ins_str, nrow(ins_df)))

ins_mod=glm(log(task_actual_time_minutes) ~ log(task_plan_time_minutes)
                                                # +num_sess
                                                +p_ID,
                                                , data=ins_df)
# print(summary(ins_mod))

return(ins_mod)
}

# Return the person_key coefficients
pe_coef=function(p_mod, num_uper)
{
# we don't want the first two, three with num_sess
c_vals=exp(coef(p_mod))[-(1:3)]
pid=as.numeric(sub("p_ID", "", names(c_vals))) # get the numeric person ID

# print(pid)
t_coef=rep(NA, num_uper)
t_coef[pid]=c_vals

return(t_coef)
}


plot_coef=function(proj_df, task_str, kind_phase)
{
# To simplify model fitting map person_key to a small integer string.
uper=unique(proj_df$person_key)
proj_df$p_ID=as.character(mapvalues(proj_df$person_key, uper, 1:length(uper)))

# The class returned by glm is a list
task_mod=alply(task_str, 1, inspect_est, proj_df)

task_mod=compact(task_mod) # Remove NULL entries from list

# Get the p_ID coefficients from each fitted model
per_coef=laply(task_mod, pe_coef, length(uper))

plot(0.1, type="n", log="y", cex=1.5,
	xaxs="i", yaxs="i",
	xlim=c(1, length(uper)+0.5), ylim=c(0.15, 4),
	xlab="Person id", ylab="Multiplicative factor")
dummy=sapply(1:nrow(per_coef),
				function(X)
				{
				points(per_coef[X, ], col=pal_col[1], cex=1.4)
				})

dummy=sapply(1:ncol(per_coef), function(X)
				{
				lines(c(X, X), range(per_coef[ , X], na.rm=TRUE), col=pal_col[2])
				})

lines(c(1, 50), c(1, 1), col="grey")
# text(ncol(per_coef)/2, 3.6, "Fitted coefficients", cex=1.2)
}

# c_phase=c("Ident", "Reqts", "HLD", "Design", "Code", "Test Devel", "Documentation")

pal_col=rainbow(2)

# List of Inspect tasks
t=unique(p615$phase_short_name)
inspect_str=t[grep("Inspect", t)]

# plot_coef(p615, inspect_str, "Inspection")
plot_coef(p615, c_phase, "Creation")

# caption{Individual multiplicative factor, for each person who made at least five estimates, combined into equation~\ref{act-est-mod}; below grey lines shows an overestimation trend, and above grey line an underestimation trend, for the seven possible inspection tasks (left) and seven possible creation tasks (right), for project 615.}

